==> nomad-vendored-1.11.0.tar.gz: nomad-vendored-1.11.0/ui/app/abilities/job.js <==
/**
 * Copyright (c) HashiCorp, Inc.
 * SPDX-License-Identifier: BUSL-1.1
 */

import AbstractAbility from './abstract';
import { computed, get } from '@ember/object';
import { or } from '@ember/object/computed';

export default class Job extends AbstractAbility {
  @or('bypassAuthorization', 'selfTokenIsManagement', 'policiesSupportRunning')
  canRun;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsRunning',
    'policiesSupportScaling'
  )
  canScale;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsReading',
    'policiesSupportReading'
  )
  canRead;

  // TODO: A person can also see all jobs if their token grants read access to all namespaces,
  // but given the complexity of namespaces and policy precedence, there isn't a good quick way
  // to confirm this.
  @or('bypassAuthorization', 'selfTokenIsManagement')
  canListAll;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'policiesSupportDispatching'
  )
  canDispatch;

  policyNamespacesIncludePermissions(policies = [], permissions = []) {
    // For each policy record, extract all policies of all namespaces
    const allNamespacePolicies = policies
      .toArray()
      .filter((policy) => get(policy, 'rulesJSON.Namespaces'))
      .map((policy) => get(policy, 'rulesJSON.Namespaces'))
      .flat()
      .map((namespace = {}) => {
        return namespace.Capabilities;
      })
      .flat()
      .compact();

    // Check for requested permissions
    return allNamespacePolicies.some((policy) => {
      return permissions.includes(policy);
    });
  }

  @computed('token.selfTokenPolicies.[]')
  get policiesSupportRunning() {
    return this.policyNamespacesIncludePermissions(
      this.token.selfTokenPolicies,
      ['submit-job']
    );
  }

  @computed('token.selfTokenPolicies.[]')
  get policiesSupportReading() {
    return this.policyNamespacesIncludePermissions(
      this.token.selfTokenPolicies,
      ['read-job']
    );
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsRunning() {
    return this.namespaceIncludesCapability('submit-job');
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsReading() {
    return this.namespaceIncludesCapability('read-job');
  }

  @computed('rulesForNamespace.@each.capabilities')
  get policiesSupportScaling() {
    return this.namespaceIncludesCapability('scale-job');
  }

  @computed('rulesForNamespace.@each.capabilities')
  get policiesSupportDispatching() {
    return this.namespaceIncludesCapability('dispatch-job');
  }
}

==> nomad-vendored-1.11.2.tar.gz: nomad-vendored-1.11.2/ui/app/abilities/job.js <==
/**
 * Copyright IBM Corp. 2015, 2025
 * SPDX-License-Identifier: BUSL-1.1
 */

import AbstractAbility from './abstract';
import { computed, get } from '@ember/object';
import { or } from '@ember/object/computed';

export default class Job extends AbstractAbility {
  @or('bypassAuthorization', 'selfTokenIsManagement', 'policiesSupportRunning')
  canRun;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsRunning',
    'policiesSupportScaling'
  )
  canScale;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsReading',
    'policiesSupportReading'
  )
  canRead;

  // TODO: A person can also see all jobs if their token grants read access to all namespaces,
  // but given the complexity of namespaces and policy precedence, there isn't a good quick way
  // to confirm this.
  @or('bypassAuthorization', 'selfTokenIsManagement')
  canListAll;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'policiesSupportDispatching'
  )
  canDispatch;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsStopping'
  )
  canStop;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsPurging'
  )
  canPurge;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsReverting'
  )
  canRevert;

  @or(
    'bypassAuthorization',
    'selfTokenIsManagement',
    'specificNamespaceSupportsRunning'
  )
  canStart;

  policyNamespacesIncludePermissions(policies = [], permissions = []) {
    // For each policy record, extract all policies of all namespaces
    const allNamespacePolicies = policies
      .toArray()
      .filter((policy) => get(policy, 'rulesJSON.Namespaces'))
      .map((policy) => get(policy, 'rulesJSON.Namespaces'))
      .flat()
      .map((namespace = {}) => {
        return namespace.Capabilities;
      })
      .flat()
      .compact();

    // Check for requested permissions
    return allNamespacePolicies.some((policy) => {
      return permissions.includes(policy);
    });
  }

  @computed('token.selfTokenPolicies.[]')
  get policiesSupportRunning() {
    return this.policyNamespacesIncludePermissions(
      this.token.selfTokenPolicies,
      ['submit-job', 'register-job']
    );
  }

  @computed('token.selfTokenPolicies.[]')
  get policiesSupportReading() {
    return this.policyNamespacesIncludePermissions(
      this.token.selfTokenPolicies,
      ['read-job']
    );
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsRunning() {
    return (
      this.namespaceIncludesCapability('submit-job') ||
      this.namespaceIncludesCapability('register-job')
    );
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsReading() {
    return this.namespaceIncludesCapability('read-job');
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsStopping() {
    return (
      this.namespaceIncludesCapability('submit-job') ||
      this.namespaceIncludesCapability('deregister-job') ||
      this.namespaceIncludesCapability('purge-job')
    );
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsPurging() {
    return (
      this.namespaceIncludesCapability('submit-job') ||
      this.namespaceIncludesCapability('purge-job')
    );
  }

  @computed('rulesForNamespace.@each.capabilities')
  get policiesSupportScaling() {
    return this.namespaceIncludesCapability('scale-job');
  }

  @computed('rulesForNamespace.@each.capabilities')
  get policiesSupportDispatching() {
    return this.namespaceIncludesCapability('dispatch-job');
  }

  @computed('rulesForNamespace.@each.capabilities')
  get specificNamespaceSupportsReverting() {
    return (
      this.namespaceIncludesCapability('submit-job') ||
      this.namespaceIncludesCapability('revert-job')
    );
  }
}
